import { AppStylesTargetBehavior } from './behaviors';
import { AppConfig } from './config';
import type { AppAssets, AppElement, AppMetadata, AppRegister, AppRegistration, AppRoute, AppStore, AppStyle, AppValidity } from './types';
/**
 * App interface.
 *
 * @example
 * ```ts
 * import { App } from '@genesislcap/foundation-shell/app';
 * ...
 * @App private app: App
 * ```
 *
 * @public
 */
export interface App extends AppAssets, AppRegister, AppValidity {
    readonly config: AppConfig;
    /**
     * @remarks
     * Register assets from either a globed dynamic import or some predefined AppAssets instance. This is a one time operation.
     * Assets had be subsequently registered and deregistered using their individual APIs.
     *
     * @param assets - An instance of AppAssets.
     */
    registerAssets(assets: Partial<AppAssets>): void;
    deregisterAssets(): void;
    hasAssets(): boolean;
}
/**
 * Default App implementation.
 * @public
 */
export declare class DefaultApp implements App {
    /** @internal */
    config: AppConfig;
    /** {@inheritDoc AppAssets.elements} */
    elements: AppElement[];
    /** {@inheritDoc AppAssets.metadata} */
    metadata: AppMetadata[];
    /** {@inheritDoc AppAssets.registrations} */
    registrations: AppRegistration[];
    /** {@inheritDoc AppAssets.routes} */
    routes: AppRoute[];
    /** {@inheritDoc AppAssets.styles} */
    styles: AppStyle[];
    /** {@inheritDoc AppAssets.stores} */
    stores: AppStore[];
    /** {@inheritDoc AppRegister.registerElementsTarget} */
    registerElementsTarget(targetId: any, predicate: any, repeatOptions?: import("@microsoft/fast-element").RepeatOptions): import("@microsoft/fast-element").ViewTemplate<any, any>;
    /** {@inheritDoc AppRegister.registerElement} */
    registerElement(asset: any): void;
    /** {@inheritDoc AppRegister.deregisterElement} */
    deregisterElement(asset: any): void;
    /** {@inheritDoc AppRegister.registerRouteCollection} */
    registerRouteCollection(routeCollection: any, predicate: any): any;
    /** {@inheritDoc AppRegister.registerRoute} */
    registerRoute(route: any): void;
    /** {@inheritDoc AppRegister.deregisterRoute} */
    deregisterRoute(route: any): void;
    /** {@inheritDoc AppRegister.registerStoreRoot} */
    registerStoreRoot(storeRoot: any, predicate: any): () => void;
    /** {@inheritDoc AppRegister.registerStore} */
    registerStore(store: any): void;
    /** {@inheritDoc AppRegister.deregisterStore} */
    deregisterStore(store: any): void;
    /** {@inheritDoc AppRegister.registerStylesTarget} */
    registerStylesTarget(targetId: any, predicate: any): AppStylesTargetBehavior;
    /** {@inheritDoc AppRegister.registerStyle} */
    registerStyle(asset: any): void;
    /** {@inheritDoc AppRegister.deregisterStyle} */
    deregisterStyle(asset: any): void;
    /** {@inheritDoc AppRegister.registerComponents} */
    registerComponents(context: any): void;
    /** {@inheritDoc AppValidity.validateMetadata} */
    isValid(context: any): boolean;
    /** {@inheritDoc App.registerAssets} */
    registerAssets(assets: any): void;
    /** {@inheritDoc App.deregisterAssets} */
    deregisterAssets(): void;
    /** {@inheritDoc App.hasAssets} */
    hasAssets(): boolean;
}
/**
 * App DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const App: import("@microsoft/fast-foundation").InterfaceSymbol<App>;
/**
 * Gets App from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getApp } from '@genesislcap/foundation-shell/app';
 * ...
 * private user = getApp();
 * ```
 *
 * @public
 */
export declare function getApp(): App;
//# sourceMappingURL=app.d.ts.map