import type { StoreRoot } from '@genesislcap/foundation-store';
import type { FoundationRoute, FoundationRouteCollection } from '@genesislcap/foundation-ui';
import type { DesignSystemModule } from '@genesislcap/foundation-utils';
import type { Behavior, ComposableStyles, RepeatOptions, ViewTemplate } from '@microsoft/fast-element';
import type { HasElement } from '@microsoft/fast-router';
/**
 * @public
 */
export type AppContext = {
    /**
     * Dynamically assigned.
     */
    env?: string;
    domain?: string;
    user?: {};
    answers?: {};
    featureFlags?: {};
};
/**
 * @public
 */
export type AppRegistrationContext = {
    /**
     * Explicitly set by app author via registerComponents.
     */
    designSystem: DesignSystemModule;
};
/**
 * @public
 * @param context - Registration context
 */
export type AppRegistration = (context: AppRegistrationContext) => void;
/**
 * @public
 */
export type HasPredicate = {
    /**
     * Optional runtime predicate to include or exclude the pbc asset.
     */
    predicate?: (context?: AppContext) => boolean;
};
/**
 * @public
 */
export type HasTarget = {
    /**
     * The target the asset is intended for, such as main, header, header-start, header-end, layout etc.
     */
    targetId: string;
};
/**
 * @remarks
 * A targetId string or an array of targetId strings
 * @public
 */
export type AppTargetId<T = string> = T | T[];
/**
 * @public
 */
export type AppStyle = HasTarget & HasPredicate & {
    /**
     * The styles to integrate.
     */
    styles: ComposableStyles | ComposableStyles[];
};
/**
 * @typeParam TSource - Optional ViewTemplate TSource interface.
 * @typeParam TParent - Optional ViewTemplate TParent interface.
 *
 * @public
 */
export type AppElement<TSource = any, TParent = any> = HasTarget & HasPredicate & {
    /**
     * The elements to integrate.
     */
    elements: ViewTemplate<TSource, TParent>;
    /**
     * Optional placement index which influences ordering.
     */
    placementIndex?: number;
    /**
     * Optional slot id of the target, such as start, end or default/omitted.
     * @privateRemarks
     * Not currently used. We could mirror https://developer.mozilla.org/en-US/docs/Web/API/Element/insertAdjacentElement#position here.
     */
    slotId?: string;
};
/**
 * @public
 */
export type AppRoute = FoundationRoute & HasElement & HasPredicate;
/**
 * @public
 */
export type AppStore<S = any> = HasPredicate & {
    store: S;
};
/**
 * @public
 */
export type AppDependencies = {
    [key: string]: string;
};
/**
 * @public
 */
export type AppDependencyPrerequisites = {
    [key: string]: string;
};
/**
 * @public
 */
export type AppMetadata = {
    name: string;
    version: string;
    description?: string;
    dependencies?: AppDependencies;
    prerequisites?: AppDependencyPrerequisites;
};
/**
 * @public
 */
export type AppValidityContext = {
    dependencies?: AppDependencies;
    prerequisites?: AppDependencyPrerequisites;
};
/**
 * @public
 */
export type AppTokenTypePredicate = (token: unknown) => boolean;
/**
 * @public
 */
export type AppRoutePredicate = (route: AppRoute, context?: AppContext) => boolean;
/**
 * @public
 */
export type AppStorePredicate = (store: AppStore, context?: AppContext) => boolean;
/**
 * @public
 */
export type AppElementPredicate = (element: AppElement, context?: AppContext) => boolean;
/**
 * @public
 */
export type AppStylePredicate = (style: AppStyle, context?: AppContext) => boolean;
/**
 * AppAssets.
 *
 * @remarks
 * Dynamically registered assets.
 *
 * @public
 */
export interface AppAssets {
    /**
     * All element tokens.
     */
    readonly elements: AppElement[];
    /**
     * All metadata tokens.
     */
    readonly metadata: AppMetadata[];
    /**
     * All App registrations.
     */
    readonly registrations: AppRegistration[];
    /**
     * All route tokens.
     */
    readonly routes: AppRoute[];
    /**
     * All style tokens.
     */
    readonly styles: AppStyle[];
    /**
     * All store tokens.
     */
    readonly stores: AppStore[];
}
/**
 * @public
 * @param routeCollection - The route collection to register pbc routes to.
 * @param predicate - An optional asset predicate.
 */
export type RegisterRouteCollection = (routeCollection: FoundationRouteCollection, predicate?: AppRoutePredicate) => (source: unknown) => void;
/**
 * @public
 * @param storeRoot - The store root to register pbc stores to.
 * @param predicate - An optional asset predicate.
 */
export type RegisterStoreRoot = (storeRoot: StoreRoot, predicate?: AppStorePredicate) => (source: unknown) => void;
/**
 * @public
 * @param targetId - A AppIntegrationTargetId.
 * @param predicate - An optional predicate.
 */
export type RegisterElementsTarget = <T = string>(targetId: AppTargetId<T>, predicate?: AppElementPredicate, repeatOptions?: RepeatOptions) => ViewTemplate;
/**
 * @public
 * @param targetId - A AppIntegrationTargetId.
 * @param predicate - An optional predicate.
 */
export type RegisterStylesTarget = <T = string>(targetId: AppTargetId<T>, predicate?: AppStylePredicate) => Behavior;
/**
 * @public
 */
export type RegisterComponents = AppRegistration;
/**
 * AppRegister.
 *
 * @remarks
 * APIs to register asset targets and assets. These can be lazily registered and deregistered.
 *
 * @public
 */
export interface AppRegister {
    registerComponents: RegisterComponents;
    registerElementsTarget: RegisterElementsTarget;
    registerElement(element: AppElement): void;
    deregisterElement(element: AppElement): void;
    registerRouteCollection: RegisterRouteCollection;
    registerRoute(route: AppRoute): void;
    deregisterRoute(route: AppRoute): void;
    registerStylesTarget: RegisterStylesTarget;
    registerStyle(style: AppStyle): void;
    deregisterStyle(asset: AppStyle): void;
    registerStoreRoot: RegisterStoreRoot;
    registerStore(style: AppStore): void;
    deregisterStore(asset: AppStore): void;
}
/**
 * AppValidity.
 *
 * @remarks
 * APIs to check app validity.
 *
 * @public
 */
export interface AppValidity {
    /**
     * @remarks
     * Monitors metadata and other data points as assets are registered and deregistered.
     *
     * @param context - A AppValidityContext.
     */
    isValid(context: AppValidityContext): boolean;
}
//# sourceMappingURL=types.d.ts.map